@@ -1,3 +1,4 @@
+import hashlib
 import os
 import shlex
 import subprocess as s
@@ -7,7 +8,10 @@ from pprint import pformat
 
 import chardet
 import click
+import pathlib
+import psutil
 import six
+import yaml
 
 from rebotics_sdk.cli.renderers import format_full_table
 from rebotics_sdk.cli.utils import pass_rebotics_context
@@ -198,16 +202,31 @@ def roles(ctx, locate):
             click.echo(key)
     elif ctx.format == 'table':
         role_configurations = []
-        for key, value in config.config.items():
+        for key, value in config.items():
             data = value
             data['role'] = key
             role_configurations.append(data)
         click.echo(format_full_table(role_configurations, 100))
 
 
+def option_default_from_configurations(conf_name):
+    class _Option(click.Option):
+        def get_default(self, ctx):
+            # try to fetch configuration from conf_name
+            if ctx.parent is None:
+                raise click.ClickException('This command is supposed to be used as sub command.')
+            role = ctx.parent.params.get('role')
+            role_config = ctx.obj.config.get(role)
+            if role_config is not None:
+                self.default = role_config.get(conf_name)
+            return super(_Option, self).get_default(ctx)
+
+    return _Option
+
+
 @click.command()
-@click.option('-h', '--host', help='App host', prompt=True)
-@click.option('-u', '--user', prompt=True, )
+@click.option('-h', '--host', help='App host', prompt=True, cls=option_default_from_configurations('host'))
+@click.option('-u', '--user', prompt=True, cls=option_default_from_configurations('username'))
 @click.option('--password', prompt=True, hide_input=True, confirmation_prompt=False)
 @pass_rebotics_context
 def configure(ctx, user, password, host):
@@ -227,15 +246,48 @@ def configure(ctx, user, password, host):
         date=datetime.now().strftime('%c'),
         **response
     )
-    click.echo(response, err=True)
-    click.echo('Saved configuration for {} in {}'.format(ctx.key, ctx.config.filepath))
+    click.echo('Saved configuration for {} in {}'.format(ctx.key, ctx.config_provider.filepath), err=True)
+    ctx.format_result(response)
 
 
 @click.command()
 @click.argument('source_file', type=click.File('r'))
 @pass_rebotics_context
 def run(ctx, source_file):
-    click.echo(exec(source_file.read(), {
+    click.echo(six.exec_(source_file.read(), {
         'ctx': ctx,
         'provider': ctx.provider,
     }))
+
+
+@main.command()
+@click.argument('codeword')
+@click.argument('param')
+def signature(codeword, param):
+    click.echo(hashlib.md5(str(codeword + param).encode('utf-8')).hexdigest())
+
+
+@main.command()
+@click.argument('compose_file', type=click.Path(exists=True, dir_okay=False))
+def clear_ports(compose_file):
+    compose_absolute_path = pathlib.Path(compose_file).absolute()
+    with s.Popen("docker-compose -f {} config".format(compose_absolute_path), stdout=s.PIPE) as proc:
+        config = proc.stdout.read().decode('utf-8')
+    conf = yaml.load(config, Loader=yaml.SafeLoader)
+
+    for service_name, value in conf['services'].items():
+        for ports_map in value.get('ports', []):
+            try:
+                published = ports_map['published']
+            except Exception as exc:
+                continue
+
+            connections = [p for p in psutil.net_connections() if p.laddr.port == published]
+
+            for con in connections:
+                click.echo("{}:{} {}".format(service_name, ports_map, con))
+                try:
+                    proc = psutil.Process(con.pid)
+                    proc.kill()
+                except Exception as exc:
+                    pass
