@@ -1,3 +1,4 @@
+import csv
 import json
 import logging
 import os
@@ -6,8 +7,10 @@ from copy import copy
 from multiprocessing import Pool
 
 import click
-from prettytable import PrettyTable
+import pandas as pd
 
+from rebotics_sdk import utils
+from rebotics_sdk.advanced.reverse_planogram import make_planogram_df
 from .common import shell, roles, configure
 from .renderers import format_processing_action_output, format_full_table
 from .tasks import task_create_processing_action_for_image, upload_preview_task, \
@@ -15,7 +18,7 @@ from .tasks import task_create_processing_action_for_image, upload_preview_task,
 from .utils import ReboticsCLIContext, pass_rebotics_context, app_dir, process_role, read_saved_role, task_runner, \
     download_file_from_dict
 from ..providers import RetailerProvider, ProviderHTTPClientException
-from ..utils import mkdir_p, get_filename_from_url
+from ..utils import mkdir_p, get_filename_from_url, download_file
 
 logger = logging.getLogger(__name__)
 
@@ -294,10 +297,6 @@ PROCESSING_STATUSES = {
 @pass_rebotics_context
 def processing_actions(ctx, store, status, page, page_size):
     """Fetches processing actions and renders them in terminal"""
-    table = PrettyTable()
-    table.field_names = ['#', 'id', 'store', 'user', 'status',
-                         'created', 'last_requeue', 'last_requeue ago', 'time in queue']
-
     if ctx.verbose:
         click.echo('Getting list of processing actions')
     try:
@@ -357,7 +356,6 @@ def assign_planogram_through_file(ctx, planogram_assign_file, deactivate):
 @click.option('-c', '--concurrency', type=int, default=4)
 @pass_rebotics_context
 def upload_brain_corp_images(ctx, braincorp_csv, store, target, concurrency):
-    import pandas as pd
     df = pd.read_csv(braincorp_csv, header=None, names=[
         'date', 'time', 'image', 'x', 'y', 'yaw', 'type', 'count', 'note',
     ])
@@ -380,7 +378,6 @@ def upload_brain_corp_images(ctx, braincorp_csv, store, target, concurrency):
 @click.option('-c', '--concurrency', type=int, default=4)
 @pass_rebotics_context
 def process_brain_corp_images(ctx, braincorp_csv, target, concurrency):
-    import pandas as pd
     df = pd.read_csv(braincorp_csv)
     pool = Pool(concurrency)
 
@@ -458,15 +455,22 @@ def user_subscribe(ctx, username, store, aisle, section):
 
 
 @api.group(invoke_without_command=True)
-@click.argument('processing_id', required=True, type=click.INT)
+@click.argument('processing_id', required=True, type=click.STRING)
 @pass_rebotics_context
 @click.pass_context
 def processing_action(click_context, ctx, processing_id):
     """Returns processing action by processing_id."""
-    setattr(ctx, 'processing_id', processing_id)
+    try:
+        processing_ids = utils.parse_id_range_string(processing_id)
+    except ValueError as exc:
+        raise click.ClickException(exc)
+
+    setattr(ctx, 'processing_id', processing_ids[0])
+    setattr(ctx, 'processing_id_string', processing_id)
+    setattr(ctx, 'processing_id_list', processing_ids)
 
     if click_context.invoked_subcommand is None:
-        result = ctx.provider.processing_action_detail(processing_id)
+        result = ctx.provider.processing_action_detail(processing_ids[0])
         format_processing_action_output([result, ], ctx.format)
 
 
@@ -577,7 +581,7 @@ def processing_action_delete(ctx):
     """Delete existing processing action by processing_id"""
     try:
         result = ctx.provider.processing_action_delete(ctx.processing_id)
-        click.echo('Successfully deactivated processing action')
+        click.echo('Successfully deactivated processing action %s', result)
     except ProviderHTTPClientException as exc:
         if ctx.verbose:
             logger.exception(exc)
@@ -585,17 +589,23 @@ def processing_action_delete(ctx):
 
 
 @processing_action.command(name='copy')
+@click.option('-s', '--store', type=click.INT)
 @pass_rebotics_context
-def processing_action_copy(ctx):
+def processing_action_copy(ctx, store):
     """Copy of the existing processing action by processing_id"""
+
     action = ctx.provider.processing_action_detail(ctx.processing_id)
 
+    store_id = action['store_id']
+    if store:
+        store_id = store
+
     inputs_id = [i['id'] for i in action['inputs']]
 
     try:
 
         result = ctx.provider.create_processing_action(
-            action['store_id'],
+            store_id,
             files=inputs_id,
             input_type=action['input_type'],
         )
@@ -620,6 +630,160 @@ def processing_action_notify_oss_report_ready(ctx, force):
         click.echo('Failed to notify OOS report ready. %s' % exc)
 
 
+@processing_action.command(name="report-image")
+@click.option('-l', '--label', type=click.Choice([
+    "unique_id",
+    "product_unique_id",
+    "user_selected_unique_id",
+    "original_unique_id"
+]), default='unique_id')
+@click.option('-t', '--target', type=click.Path(), default='.')
+@pass_rebotics_context
+def processing_action_report_image(ctx, label, target):
+    image_url = ctx.provider.build_url("/api/v4/processing/actions/{id}/report_image/".format(id=ctx.processing_id))
+    if ctx.verbose:
+        click.echo("URL: {}".format(image_url))
+
+    p = '{}_{}.jpeg'.format(ctx.processing_id, label)
+    if os.path.isdir(target):
+        p = os.path.join(target, p)
+
+    filepath = download_file(image_url, p, label=label)
+    click.echo('Saved to {}'.format(filepath))
+
+
+@processing_action.command(name='realogram-report')
+@click.option('-c', '--concurrency', default=4, help='Define how many parallel workers will be used to access API')
+@click.option('-d', '--drop-duplicates', is_flag=True, default=False, help="Doesn't work for now")
+@click.option('-t', '--target', type=click.Path(file_okay=False), default='.', help='Define specific folder where'
+                                                                                    ' the report files will be saved')
+@pass_rebotics_context
+def processing_action_realogram_report(ctx, concurrency, drop_duplicates, target):
+    """
+    Function to download processing actions and extract preview from them.
+    It will generate three files: scans.csv, realogram_report.csv, and oos_report.csv from given range or
+    comma-separated processing action ids
+
+    Typical usage is:
+
+        retailer processing-action 1000-1005,1010-1015,1017 realogram-report
+
+
+    Be careful with running the report two times, as it will override the data in folder
+     if you have not specified the target folder
+    """
+    pool = Pool(concurrency)
+    processing_action_ids = ctx.processing_id_list
+    click.echo('downloading processing actions: {}'.format(processing_action_ids))
+    pacs = pool.starmap(task_download_processing_action, [(ctx, id_) for id_ in processing_action_ids])
+    processing_actions = [p for p in pacs if p is not None]
+
+    click.echo('Download complete! Generating files...')
+
+    df_scans = pd.DataFrame(dtype=object)
+    df_oos = pd.DataFrame(dtype=object)
+    df_realogram = pd.DataFrame(dtype=object)
+
+    for processing_action in processing_actions:
+        df_bay_reports = pd.DataFrame(processing_action['shelf_bays'], columns=[
+            'id',
+            'store_planogram_id',
+            'aisle',
+            'section'
+        ], dtype=object)
+        realogram = processing_action['items']
+
+        df_realogram_pac = pd.DataFrame(realogram, columns=[
+            'unique_id',
+            'on_shelf_position', 'display_shelf', 'shelf',
+            'position',
+            'left', 'top', 'right', 'bottom',
+            'shelf_bay_id',
+        ], dtype=object)
+
+        df_realogram_pac['processing_action'] = processing_action['id']
+        df_realogram_pac['store_id'] = processing_action['store_id']
+        df_realogram_pac['store_name'] = processing_action['store_name']
+        df_realogram_pac['store_number'] = processing_action['store']['store_number']
+        df_realogram_pac['created'] = processing_action['created']
+
+        df_realogram_pac = df_realogram_pac.merge(
+            df_bay_reports, left_on='shelf_bay_id', right_on='id'
+        )
+        df_realogram = df_realogram.append(df_realogram_pac)
+
+        actions = processing_action.get('report_actions', [])
+        oos = [a for a in actions if a['action'] == 'ACTION_ADD']
+        df_oos_processing_action = pd.DataFrame(oos, columns=[
+            'plu', 'from', 'to', 'from_aisle', 'to_aisle',
+            'shelf_bay_id',
+        ], dtype=object)
+
+        df_oos_processing_action['processing_action'] = processing_action['id']
+        df_oos_processing_action['store_id'] = processing_action['store_id']
+        df_oos_processing_action['store_name'] = processing_action['store_name']
+        df_oos_processing_action['store_number'] = processing_action['store'].get('store_number')
+        df_oos_processing_action['created'] = processing_action['created']
+        df_oos_processing_action = df_oos_processing_action.merge(
+            df_bay_reports, left_on='shelf_bay_id', right_on='id'
+        )
+        df_oos = df_oos.append(df_oos_processing_action)
+
+        df_pac = pd.DataFrame([processing_action, ], columns=[
+            'id', 'store_name', 'store_id', 'created', 'used_in_report',
+            'category_name', 'category_number',
+            'store_planogram_id', 'aisle', 'section',
+            'download_report_url',
+        ], index=['id'], dtype=object)
+        df_pac['user'] = processing_action['user'].get('username', '')
+        df_pac['compliance_rate'] = processing_action['compliance_rates'].get('compliance', '0')
+        df_pac['store_number'] = processing_action['store'].get('store_number')
+        df_scans = df_scans.append(df_pac)
+
+    # if drop_duplicates:
+    #     TODO: here should be user definable date or splittable. Extend the API
+    # df_oos = df_oos.drop_duplicates(['store_number', 'plu', 'created'])
+
+    dump_path = os.path.join(target, 'reporting_{}'.format(ctx.processing_id_string))
+    mkdir_p(dump_path)
+
+    df_realogram.to_csv(
+        os.path.join(dump_path, 'realogram_report.csv'),
+        quoting=csv.QUOTE_ALL,
+        index=False
+    )
+    df_oos.to_csv(
+        os.path.join(dump_path, 'oos_report.csv'),
+        quoting=csv.QUOTE_ALL,
+        index=False
+    )
+    df_scans.to_csv(
+        os.path.join(dump_path, 'scans.csv'),
+        quoting=csv.QUOTE_ALL,
+        index=False
+    )
+
+    click.echo('Written to {}'.format(dump_path))
+
+
+@processing_action.command()
+@click.option('-t', '--target', type=click.Path(file_okay=False), default='.', help='Define specific folder where'
+                                                                                    ' the reverse planogram file will be saved')
+@pass_rebotics_context
+def reverse_planogram(ctx, target):
+    if ctx.verbose:
+        click.echo("Downloading scan with id #{}".format(ctx.processing_id))
+
+    scan = ctx.provider.processing_action_detail(ctx.processing_id)
+    df = make_planogram_df(scan)
+
+    p = 'reverse_planogram_{}.csv'.format(ctx.processing_id)
+    if os.path.isdir(target):
+        p = os.path.join(target, p)
+    df.to_csv(p, index=False, quoting=csv.QUOTE_ALL)
+    click.echo("Saved into {}".format(p))
+
+
 api.add_command(shell, 'shell')
 api.add_command(roles, 'roles')
 api.add_command(configure, 'configure')
